@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2017
+# Copyright (C) 2015-2018
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,7 +17,7 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 # TODO: Remove allow_edited
-""" This module contains the RegexHandler class """
+"""This module contains the RegexHandler class."""
 
 import re
 import warnings
@@ -29,11 +29,11 @@ from .handler import Handler
 
 
 class RegexHandler(Handler):
-    """
-    Handler class to handle Telegram updates based on a regex. It uses a
-    regular expression to check text messages. Read the documentation of the
-    ``re`` module for more information. The ``re.match`` function is used to
-    determine if an update should be handled by this handler.
+    """Handler class to handle Telegram updates based on a regex.
+
+    It uses a regular expression to check text messages. Read the documentation of the ``re``
+    module for more information. The ``re.match`` function is used to determine if an update should
+    be handled by this handler.
 
     Attributes:
         pattern (:obj:`str` | :obj:`Pattern`): The regex pattern.
@@ -91,6 +91,7 @@ class RegexHandler(Handler):
 
     Raises:
         ValueError
+
     """
 
     def __init__(self,
@@ -133,32 +134,32 @@ class RegexHandler(Handler):
         self.edited_updates = edited_updates
 
     def check_update(self, update):
-        """
-        Determines whether an update should be passed to this handlers :attr:`callback`.
+        """Determines whether an update should be passed to this handlers :attr:`callback`.
 
         Args:
             update (:class:`telegram.Update`): Incoming telegram update.
 
         Returns:
             :obj:`bool`
+
         """
         if not isinstance(update, Update) and not update.effective_message:
             return False
-        if any([(self.message_updates and update.message),
-                (self.edited_updates and update.edited_message),
-                (self.channel_post_updates and update.channel_post)]) and \
+        if any([self.message_updates and update.message,
+                self.edited_updates and (update.edited_message or update.edited_channel_post),
+                self.channel_post_updates and update.channel_post]) and \
                 update.effective_message.text:
             match = re.match(self.pattern, update.effective_message.text)
             return bool(match)
         return False
 
     def handle_update(self, update, dispatcher):
-        """
-        Send the update to the :attr:`callback`.
+        """Send the update to the :attr:`callback`.
 
         Args:
             update (:class:`telegram.Update`): Incoming telegram update.
             dispatcher (:class:`telegram.ext.Dispatcher`): Dispatcher that originated the Update.
+
         """
 
         optional_args = self.collect_optional_args(dispatcher, update)
